from mssqlctl.cli.core.clients.cli_client import CliClient
from http import HTTPStatus
from mssqlctl.cli.core.password import PasswordManager
from mssqlctl.cli.core.deploy import display
from urllib3.exceptions import NewConnectionError, MaxRetryError

from mssqlctl.cli.core.logging import get_logger
import mssqlctl.cli.core.deploy as util
import mssqlctl.cli.core.kubernetes as kubernetes_util
import time

CONNECTION_RETRY_ATTEMPTS = 12
DELETE_CLUSTER_TIMEOUT_SECONDS = 300
RETRY_INTERVAL = 5
UPDATE_INTERVAL = (15 * 60) / RETRY_INTERVAL
logger = get_logger(__name__)


def beget(_):
    """Client factory"""
    return ControlClientMixin()


def beget_no_check_auth(_):
    """Client factory - no check on authentication"""
    return ControlClientMixin(False)


def beget_no_check_auth_or_eula(_):
    """Client factory - no check on authentication or eula"""
    return ControlClientMixin(False, False)


class ControlClientMixin(CliClient):
    def __init__(self, check_auth=True, check_eula=True):
        super(ControlClientMixin, self).__init__(check_auth, check_eula)
        self.cluster_name = None

    def control_create(self, cluster_name, config_object):
        self.cluster_name = cluster_name
        PasswordManager.check_password_validity("cluster")
        self.cluster_object = config_object
        self.cluster_secrets = dict()

        # Overwrite cluster name if it is specified on the command line.
        #
        if self.cluster_name is not None and self.cluster_name != "":
            self.cluster_object["metadata"]["name"] = self.cluster_name
        else:
            self.cluster_name = self.cluster_object["metadata"]["name"]

        self.apis.control_deploy.create_control_plane(self.cluster_object, self.cluster_name)

    def control_delete(self, cluster_name):
        """
                Delete a cluster
                """
        self.cluster_name = cluster_name
        display("Deleting control '%s'." % self.cluster_name)

        if not util.retry(kubernetes_util.namespace_exists, self.cluster_name,
                          retry_count=CONNECTION_RETRY_ATTEMPTS, retry_delay=RETRY_INTERVAL,
                          retry_method="check if namespace exists",
                          retry_on_exceptions=(NewConnectionError, MaxRetryError)):
            display("Control '%s' doesn't exist" % self.cluster_name)
            return

        # Try to delete the cluster
        #
        i = 1
        resources_are_deleted = False
        http_status = None
        cluster_is_empty = False
        while not cluster_is_empty:

            time.sleep(RETRY_INTERVAL)

            if not resources_are_deleted:
                #  Try to delete the remaining resources in the cluster
                #
                (resources_are_deleted, http_status) = util.retry(kubernetes_util.delete_cluster_resources,
                                                                  self.cluster_name,
                                                                  retry_count=CONNECTION_RETRY_ATTEMPTS,
                                                                  retry_delay=RETRY_INTERVAL,
                                                                  retry_method="delete cluster resources",
                                                                  retry_on_exceptions=(
                                                                  NewConnectionError, MaxRetryError))

                if (http_status == HTTPStatus.FORBIDDEN):
                    break

            # Check if the cluster is empty
            #
            cluster_is_empty = util.retry(kubernetes_util.namespace_resources_are_deleted, self.cluster_name,
                                          retry_count=CONNECTION_RETRY_ATTEMPTS,
                                          retry_delay=RETRY_INTERVAL, retry_method="delete namespace resources",
                                          retry_on_exceptions=(NewConnectionError, MaxRetryError))

            if i * RETRY_INTERVAL > DELETE_CLUSTER_TIMEOUT_SECONDS:
                logger.warn("Control is not empty after %d minutes." % (DELETE_CLUSTER_TIMEOUT_SECONDS / 60))
                break

            i = i + 1
            time.sleep(RETRY_INTERVAL)

        if cluster_is_empty:
            display("Control deleted successfully.")
        else:
            raise Exception("Failed to delete control.")
