# ------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ------------------------------------------------------------------------------

"""Command definitions for `control`."""

from mssqlctl.cli.commands.control.exceptions import ControlError
from mssqlctl.cli.core.constants import (
    CONFIG_DIR, HELP_DIR, CONTROL_CONFIG_FILENAME
)
from mssqlctl.cli.core.util import (eula_prompt)
from mssqlctl.cli.core.exceptions import CliError
from mssqlctl.cli.core.configuration import Configuration
from knack.prompting import NoTTYException
from mssqlctl.cli.core.prompt import (prompt_for_input, prompt_for_choice, prompt_y_n, prompt_pass, prompt)
from humanfriendly import AutomaticSpinner
from jsonpath_ng.ext import parse

import mssqlctl.cli.core.deploy as util
import os
import json
import sys


def control_create(client, name=None, config_profile=None, accept_eula=None, node_label=None, force=None):
    """
    If an argument is not provided, the user will be prompted for the needed
    values NoTTY Scenario: provide a config_profile, accept_eula
    :param client:
    :param name:
    :param config_profile:
    :param accept_eula:
    :param node_label:
    :param force:
    :return:
    """
    try:
        stdout = client.stdout

        # -- Check if custom config profile exists --
        if config_profile and not os.path.isdir(config_profile):
            # Check in the default directory
            config_profile = os.path.join(CONFIG_DIR, config_profile)
            if not os.path.isdir(config_profile):
                raise ValueError('Config profile {0} does not exist.'.
                                 format(os.path.basename(config_profile)))

        # -- enforce EULA --
        client.profile.accept_eula = eula_prompt(client, accept_eula, True, force)

        # -- Configuration Directory --
        if not config_profile and not force:
            profiles = util.get_config_profiles()

            # Prompt the user for a choice between configs
            stdout('Please choose a deployment configuration: ')
            stdout('To see more info please exit create and use command '
                   '[bdc config list -c <config_profile>]')
            choices = util.get_config_display_names(profiles)
            result = prompt_for_choice(choices, choices[1])

            config_profile = os.path.join(CONFIG_DIR, profiles[result.lower()])
        elif not config_profile and force:
            raise ValueError('Please specify a configuration profile as an argument via '
                             '\'-c <configuration profile>\' when using --force')

        # -- Check Kubectl Context --
        util.check_and_set_kubectl_context()

        # -- Required Environment Variables --
        if not force and sys.stdin.isatty():
            util.read_environment_variables('cluster')
        else:
            util.check_environment_variables('cluster')

        # -- Read json into python dictionary --
        config_object = util.read_config(config_profile, CONTROL_CONFIG_FILENAME)
        # -- Get help documentation for missing values --
        help_object = util.read_config(HELP_DIR, CONTROL_CONFIG_FILENAME)
        # -- If security is enabled, check for required security environment variables --
        util.security_check(stdout, force, config_object)
        # -- Check for missing values in the config object --
        util.check_missing(stdout, force, config_object, help_object, CONTROL_CONFIG_FILENAME)
        # -- Check if control config is valid
        util.control_config_check(stdout, config_object)

        try:
            # -- Add cluster nodes label to config if included --
            if node_label is not None:
                config_object["spec"]["nodeLabel"] = node_label
            config_object['spec'] = {'controlPlane': {'spec': config_object['spec']}}
        except Exception as e:
            raise ValueError('Invalid config profile: {0}'.format(e))

        # -- attempt to create cluster --
        stdout('NOTE: Cluster creation can take a significant amount of time '
               'depending on')
        stdout('configuration, network speed, and the number of nodes in the '
               'cluster.')
        stdout('')

        client.control_create(name, config_object)
    except NoTTYException:
        raise CliError('Please specify [--accept-eula, --config-file] in '
                       'non-interactive mode.')
    except ValueError as e:
        raise CliError(e)
    except ControlError as e:
        raise CliError(e)
    except Exception as e:
        raise CliError(e)


def control_delete(client, name, force=None):
    """
    Deletes the control plane - requires kube config and env var
    :param client:
    :param name:
    :param force:
    :return:
    """
    try:
        stdout = client.stdout
        stdout('')
        stdout('This operation will delete everything inside of control "{}"'.format(name))
        stdout('which includes the SQL Server containers, Kubernetes secrets and services,')
        stdout('and HDFS containers. Data stored on persistent volumes will get deleted if')
        stdout('the storage class reclaim policy is set to delete/recycle.')
        stdout('')

        if not force:
            yes = prompt_y_n('Do you want to continue with deleting '
                             'the control "{}"?'.format(name))
        else:
            yes = force  # yes|no for no-tty

        if yes != 'yes' and yes is not True:
            msg = 'Control not deleted. Exiting...'
            return msg

        # -- Check Kubectl Context --
        util.check_and_set_kubectl_context()

        # -- attempt to delete cluster --
        client.control_delete(name)
    except NoTTYException:
        raise CliError('Please specify `--force` in non-interactive mode.')
    except ControlError as e:
        raise CliError(e)
    except Exception as e:
        raise CliError(e)


